# This code based on introspection of TKP "catalog" database.

from django.db import models
import astro # local routines
import numpy, math

class Classification(models.Model):
    """
    This model effectively unused; just a placeholder for now.
    """
    classid = models.IntegerField(primary_key=True)
    type = models.IntegerField()
    class Meta:
        db_table = u'classification'

class Spectralparameters(models.Model):
    """
    Parameters for spectral modelling of sources.
    """
    spectral_paramsid = models.IntegerField(primary_key=True)
    param_n = models.IntegerField()
    f74 = models.FloatField(null=True, blank=True)
    alpha_gt_300 = models.FloatField(null=True, blank=True)
    f_ext = models.FloatField(null=True, blank=True)
    param_a = models.FloatField(null=True, blank=True)
    param_b = models.FloatField(null=True, blank=True)
    param_c = models.FloatField(null=True, blank=True)
    param_d = models.FloatField(null=True, blank=True)
    param_rms_diff = models.FloatField(null=True, blank=True)
    class Meta:
        db_table = u'spectralparameters'
        

class SourceManager(models.Manager):
    """
    This manager object enables us to filter QuerySets by position. Sweet!

    But, uh, pardon my SQL...
    """
    def near(self, ra, dec, theta):
        return self.extra(
            where=['COS(RADIANS(ra))*COS(RADIANS(decl)) * COS(RADIANS(%s))*COS(RADIANS(%s)) + SIN(RADIANS(ra))*COS(RADIANS(decl)) * SIN(RADIANS(%s))*COS(RADIANS(%s)) + SIN(RADIANS(decl)) * SIN(RADIANS(%s)) > COS(RADIANS(%s))'],
            params=[ra, dec, ra, dec, dec, theta]
        )

class Sources(models.Model):
    """
    Sources, with methods for estimating fluxes at different frequencies.
    """
    srcid = models.IntegerField(primary_key=True)
    src_name = models.CharField(max_length=150, blank=True)
    cattype = models.CharField(max_length=3, blank=True)
    class_field = models.ForeignKey(Classification, null=True, blank=True, db_column="class_id")
    zone = models.IntegerField()
    ra = models.FloatField()
    decl = models.FloatField()
    ra_err = models.FloatField()
    decl_err = models.FloatField()
    x = models.FloatField()
    y = models.FloatField()
    z = models.FloatField()
    spectral_params = models.ForeignKey(Spectralparameters, null=True, blank=True)

    objects = SourceManager()

    class Meta:
        db_table = u'sources'

    def __str__(self):
        return str(self.src_name)

    def helmboldt_db(self, freq_MHz):
        """
        Helmboldt fluxes using stored data in database.
        """
        from django.db import connection
        cursor = connection.cursor()
        cursor.execute("SELECT alpha_gt_300 * LOG10(%s/74) + LOG10(f_ext) AS 'linear: log F_{nu} (Jy)', CASE WHEN param_a IS NULL THEN NULL ELSE param_a + IFNULL(param_b, 0) * LOG10(%s/74) + IFNULL(param_c, 0) * EXP(IFNULL(param_d, 0) * LOG10(%s/74)) END AS 'Kuehr' FROM spectralparameters WHERE spectral_paramsid = %s", [freq_MHz, freq_MHz, freq_MHz, self.spectral_params.spectral_paramsid])
        row = cursor.fetchone()
        return row

    def linear(self, freq_Mhz):
        """
        Use local Python code rather than the DB for calculation.
        """
        return (self.spectral_params.alpha_gt_300 or 0) * numpy.log10(freq_Mhz/74.0) + numpy.log10(self.spectral_params.f_ext or 1)

    def kuehr(self, freq_Mhz):
        """
        Use local Python code rather than the DB for calculation.
        """
        if self.spectral_params.param_a:
            return self.spectral_params.param_a + (self.spectral_params.param_b or 0) * numpy.log10(freq_Mhz/74.0) + (self.spectral_params.param_c or 0) * numpy.exp((self.spectral_params.param_d or 0) * numpy.log10(freq_Mhz/74.0))

    def within(self, ra, dec, theta):
        """
        Return true if our RA & dec is within theta degrees of the given
        location.
        """
        # In theory, we should use self.x/y/z, but Bart's definition and mine
        # are different & I don't currently have time to figure out why.
        local = numpy.array(astro.cartesian(self.ra, self.decl))
        target = numpy.array(astro.cartesian(ra, dec))
        if numpy.dot(local, target) > numpy.cos(math.radians(theta)):
            return True
        else:
            return False
